# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SubscriptionListParams"]


class SubscriptionListParams(TypedDict, total=False):
    brand_id: str
    """filter by Brand id"""

    created_at_gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Get events after this created time"""

    created_at_lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Get events created before this time"""

    customer_id: str
    """Filter by customer id"""

    page_number: int
    """Page number default is 0"""

    page_size: int
    """Page size default is 10 max is 100"""

    status: Literal["pending", "active", "on_hold", "paused", "cancelled", "failed", "expired"]
    """Filter by status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DiscountListParams"]


class DiscountListParams(TypedDict, total=False):
    page_number: int
    """Page number (default = 0)."""

    page_size: int
    """Page size (default = 10, max = 100)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["BrandUpdateImagesResponse"]


class BrandUpdateImagesResponse(BaseModel):
    image_id: str
    """UUID that will be used as the image identifier/key suffix"""

    url: str
    """Presigned URL to upload the image"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .time_interval import TimeInterval

__all__ = ["LicenseKeyDurationParam"]


class LicenseKeyDurationParam(TypedDict, total=False):
    count: Required[int]

    interval: Required[TimeInterval]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["DisputeListParams"]


class DisputeListParams(TypedDict, total=False):
    created_at_gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Get events after this created time"""

    created_at_lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Get events created before this time"""

    customer_id: str
    """Filter by customer_id"""

    dispute_stage: Literal["pre_dispute", "dispute", "pre_arbitration"]
    """Filter by dispute stage"""

    dispute_status: Literal[
        "dispute_opened",
        "dispute_expired",
        "dispute_accepted",
        "dispute_cancelled",
        "dispute_challenged",
        "dispute_won",
        "dispute_lost",
    ]
    """Filter by dispute status"""

    page_number: int
    """Page number default is 0"""

    page_size: int
    """Page size default is 10 max is 100"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["CustomerPortalSession"]


class CustomerPortalSession(BaseModel):
    link: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TimeInterval"]

TimeInterval: TypeAlias = Literal["Day", "Week", "Month", "Year"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel
from .currency import Currency
from .intent_status import IntentStatus
from .customer_limited_details import CustomerLimitedDetails

__all__ = ["PaymentListResponse"]


class PaymentListResponse(BaseModel):
    brand_id: str

    created_at: datetime

    currency: Currency

    customer: CustomerLimitedDetails

    digital_products_delivered: bool

    metadata: Dict[str, str]

    payment_id: str

    total_amount: int

    payment_method: Optional[str] = None

    payment_method_type: Optional[str] = None

    status: Optional[IntentStatus] = None

    subscription_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["AddonUpdateImagesResponse"]


class AddonUpdateImagesResponse(BaseModel):
    image_id: str

    url: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["DisputeStage"]

DisputeStage: TypeAlias = Literal["pre_dispute", "dispute", "pre_arbitration"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from dodopayments import DodoPayments, AsyncDodoPayments
from dodopayments.types import (
    LicenseKeyInstance,
    LicenseValidateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLicenses:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_activate(self, client: DodoPayments) -> None:
        license = client.licenses.activate(
            license_key="license_key",
            name="name",
        )
        assert_matches_type(LicenseKeyInstance, license, path=["response"])

    @parametrize
    def test_raw_response_activate(self, client: DodoPayments) -> None:
        response = client.licenses.with_raw_response.activate(
            license_key="license_key",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        license = response.parse()
        assert_matches_type(LicenseKeyInstance, license, path=["response"])

    @parametrize
    def test_streaming_response_activate(self, client: DodoPayments) -> None:
        with client.licenses.with_streaming_response.activate(
            license_key="license_key",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            license = response.parse()
            assert_matches_type(LicenseKeyInstance, license, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_deactivate(self, client: DodoPayments) -> None:
        license = client.licenses.deactivate(
            license_key="license_key",
            license_key_instance_id="license_key_instance_id",
        )
        assert license is None

    @parametrize
    def test_raw_response_deactivate(self, client: DodoPayments) -> None:
        response = client.licenses.with_raw_response.deactivate(
            license_key="license_key",
            license_key_instance_id="license_key_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        license = response.parse()
        assert license is None

    @parametrize
    def test_streaming_response_deactivate(self, client: DodoPayments) -> None:
        with client.licenses.with_streaming_response.deactivate(
            license_key="license_key",
            license_key_instance_id="license_key_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            license = response.parse()
            assert license is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_validate(self, client: DodoPayments) -> None:
        license = client.licenses.validate(
            license_key="2b1f8e2d-c41e-4e8f-b2d3-d9fd61c38f43",
        )
        assert_matches_type(LicenseValidateResponse, license, path=["response"])

    @parametrize
    def test_method_validate_with_all_params(self, client: DodoPayments) -> None:
        license = client.licenses.validate(
            license_key="2b1f8e2d-c41e-4e8f-b2d3-d9fd61c38f43",
            license_key_instance_id="lki_123",
        )
        assert_matches_type(LicenseValidateResponse, license, path=["response"])

    @parametrize
    def test_raw_response_validate(self, client: DodoPayments) -> None:
        response = client.licenses.with_raw_response.validate(
            license_key="2b1f8e2d-c41e-4e8f-b2d3-d9fd61c38f43",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        license = response.parse()
        assert_matches_type(LicenseValidateResponse, license, path=["response"])

    @parametrize
    def test_streaming_response_validate(self, client: DodoPayments) -> None:
        with client.licenses.with_streaming_response.validate(
            license_key="2b1f8e2d-c41e-4e8f-b2d3-d9fd61c38f43",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            license = response.parse()
            assert_matches_type(LicenseValidateResponse, license, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncLicenses:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_activate(self, async_client: AsyncDodoPayments) -> None:
        license = await async_client.licenses.activate(
            license_key="license_key",
            name="name",
        )
        assert_matches_type(LicenseKeyInstance, license, path=["response"])

    @parametrize
    async def test_raw_response_activate(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.licenses.with_raw_response.activate(
            license_key="license_key",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        license = await response.parse()
        assert_matches_type(LicenseKeyInstance, license, path=["response"])

    @parametrize
    async def test_streaming_response_activate(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.licenses.with_streaming_response.activate(
            license_key="license_key",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            license = await response.parse()
            assert_matches_type(LicenseKeyInstance, license, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_deactivate(self, async_client: AsyncDodoPayments) -> None:
        license = await async_client.licenses.deactivate(
            license_key="license_key",
            license_key_instance_id="license_key_instance_id",
        )
        assert license is None

    @parametrize
    async def test_raw_response_deactivate(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.licenses.with_raw_response.deactivate(
            license_key="license_key",
            license_key_instance_id="license_key_instance_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        license = await response.parse()
        assert license is None

    @parametrize
    async def test_streaming_response_deactivate(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.licenses.with_streaming_response.deactivate(
            license_key="license_key",
            license_key_instance_id="license_key_instance_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            license = await response.parse()
            assert license is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_validate(self, async_client: AsyncDodoPayments) -> None:
        license = await async_client.licenses.validate(
            license_key="2b1f8e2d-c41e-4e8f-b2d3-d9fd61c38f43",
        )
        assert_matches_type(LicenseValidateResponse, license, path=["response"])

    @parametrize
    async def test_method_validate_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        license = await async_client.licenses.validate(
            license_key="2b1f8e2d-c41e-4e8f-b2d3-d9fd61c38f43",
            license_key_instance_id="lki_123",
        )
        assert_matches_type(LicenseValidateResponse, license, path=["response"])

    @parametrize
    async def test_raw_response_validate(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.licenses.with_raw_response.validate(
            license_key="2b1f8e2d-c41e-4e8f-b2d3-d9fd61c38f43",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        license = await response.parse()
        assert_matches_type(LicenseValidateResponse, license, path=["response"])

    @parametrize
    async def test_streaming_response_validate(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.licenses.with_streaming_response.validate(
            license_key="2b1f8e2d-c41e-4e8f-b2d3-d9fd61c38f43",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            license = await response.parse()
            assert_matches_type(LicenseValidateResponse, license, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["BrandRetrieveResponse"]


class BrandRetrieveResponse(BaseModel):
    brand_id: str

    business_id: str

    enabled: bool

    statement_descriptor: str

    verification_enabled: bool

    verification_status: Literal["Success", "Fail", "Review", "Hold"]

    description: Optional[str] = None

    image: Optional[str] = None

    name: Optional[str] = None

    reason_for_hold: Optional[str] = None
    """Incase the brand verification fails or is put on hold"""

    support_email: Optional[str] = None

    url: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["WebhookEventType"]

WebhookEventType: TypeAlias = Literal[
    "payment.succeeded",
    "payment.failed",
    "payment.processing",
    "payment.cancelled",
    "refund.succeeded",
    "refund.failed",
    "dispute.opened",
    "dispute.expired",
    "dispute.accepted",
    "dispute.cancelled",
    "dispute.challenged",
    "dispute.won",
    "dispute.lost",
    "subscription.active",
    "subscription.renewed",
    "subscription.on_hold",
    "subscription.paused",
    "subscription.cancelled",
    "subscription.failed",
    "subscription.expired",
    "subscription.plan_changed",
    "license_key.created",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["CustomerPortalCreateParams"]


class CustomerPortalCreateParams(TypedDict, total=False):
    send_email: bool
    """If true, will send link to user."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["LicenseKeyUpdateParams"]


class LicenseKeyUpdateParams(TypedDict, total=False):
    activations_limit: Optional[int]
    """
    The updated activation limit for the license key. Use `null` to remove the
    limit, or omit this field to leave it unchanged.
    """

    disabled: Optional[bool]
    """
    Indicates whether the license key should be disabled. A value of `true` disables
    the key, while `false` enables it. Omit this field to leave it unchanged.
    """

    expires_at: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """
    The updated expiration timestamp for the license key in UTC. Use `null` to
    remove the expiration date, or omit this field to leave it unchanged.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["LicenseKeyStatus"]

LicenseKeyStatus: TypeAlias = Literal["active", "expired", "disabled"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["WebhookEventListParams"]


class WebhookEventListParams(TypedDict, total=False):
    created_at_gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Get events after this created time"""

    created_at_lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Get events created before this time"""

    limit: int
    """Min : 1, Max : 100, default 10"""

    object_id: str
    """
    Get events history of a specific object like payment/subscription/refund/dispute
    """

    page_number: int
    """Page number default is 0"""

    page_size: int
    """Page size default is 10 max is 100"""

    webhook_event_id: str
    """Filter by webhook event id"""

    webhook_id: str
    """Filter by webhook destination"""
